package com.taskttl.ui.screen

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Button
import androidx.compose.material3.IconButton
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import androidx.navigation.NavGraphBuilder
import androidx.navigation.NavOptions
import androidx.navigation.compose.composable
import com.taskttl.R
import com.taskttl.data.constant.Constant
import com.taskttl.data.contract.AddEffect
import com.taskttl.data.contract.AddEvent
import com.taskttl.ui.components.FromLabel
import com.taskttl.ui.viewmodel.AddViewModel
import com.taskttl.ui.viewmodel.CollectSideEffect
import com.taskttl.utils.DateUtil
import com.taskttl.utils.ToastUtil


/** 首页 */
const val ADD_ROUTE = "add"
const val TASK_DETAIL_ROUTE = "task_detail"

fun NavController.navigateToAdd(navOptions: NavOptions) = navigate(ADD_ROUTE, navOptions)


fun NavGraphBuilder.addScreen(snackBarHostState: SnackbarHostState) {
    composable(route = ADD_ROUTE) {
        AddRoute(snackBarHostState)
    }
}

@Composable
fun AddRoute(
    snackBarHostState: SnackbarHostState,
    id: Int = 0,
    addViewModel: AddViewModel = hiltViewModel()
) {
    val state by addViewModel.uiState.collectAsState()

    // 使用 remember 保存状态
    var titleValid by remember { mutableStateOf(false) }

    var date by remember { mutableStateOf("") }
    var contentValid by remember { mutableStateOf(false) }
    var categoryData by remember { mutableIntStateOf(0) }

    LaunchedEffect(id) {
        if (id > 0) {
            addViewModel.sendEvent(AddEvent.LoadTask(id))
        }
    }

    addViewModel.CollectSideEffect { sideEffect ->
        when (sideEffect) {
            is AddEffect.AddSuccess -> {
                ToastUtil.showLongToast(sideEffect.message)
//                navController.popBackStack()
            }
        }
    }

    Box(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(0xFFF1F5F9))
    ) {
        Image(
            painter = painterResource(id = R.mipmap.add_header),
            modifier = Modifier
                .fillMaxWidth()
                .height(96.dp),
            contentScale = ContentScale.Crop,
            contentDescription = ""
        )

        Text(
            stringResource(R.string.add_new_task),
            fontSize = 30.sp,
            fontWeight = FontWeight.Bold,
            color = Color.White,
            modifier = Modifier
                .align(Alignment.TopCenter)
                .padding(top = 36.dp)
        )

        Column(
            modifier = Modifier
                .padding(start = 16.dp, end = 16.dp, top = 96.dp)
                .fillMaxSize()
        ) {
            val colors = TextFieldDefaults.colors(
                focusedTextColor = Color.Unspecified,
                unfocusedTextColor = Color.Unspecified,
                disabledTextColor = Color.Gray,
                errorTextColor = Color.Unspecified,
                focusedContainerColor = Color.White,
                unfocusedContainerColor = Color.White,
                disabledContainerColor = Color.Unspecified,
                errorContainerColor = Color.Unspecified,
                cursorColor = Color.Unspecified,
                errorCursorColor = Color.Unspecified,
                focusedIndicatorColor = Color.Transparent,
                unfocusedIndicatorColor = Color.Transparent,
                errorIndicatorColor = Color.Red,
            )
            FromLabel("Task Title")
            TextField(
                placeholder = {
                    Text(
                        "Task Title", fontSize = 16.sp, modifier = Modifier.alpha(0.7f)
                    )
                },
                value = state.taskEntity.title,
                onValueChange = {
                    addViewModel.sendEvent(AddEvent.UpdateTitle(it))
                    titleValid = it.isNotEmpty()
                },
                keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text),
                modifier = Modifier
                    .fillMaxWidth()
                    .height(55.dp),
                shape = RoundedCornerShape(6.dp),
                colors = colors,
                isError = !titleValid
            )
            if (!titleValid) {
                Text(text = "Please enter valid text", color = Color.Red)
            }
            Row(
                modifier = Modifier
                    .padding(top = 24.dp)
                    .height(48.dp),
                verticalAlignment = Alignment.CenterVertically
            ) {
                FromLabel("Category", 0.dp)
                Spacer(modifier = Modifier.width(24.dp))
                Constant.imageList.forEachIndexed { index, item ->
                    var modifier = Modifier.size(48.dp)

                    if (categoryData == index) {
                        modifier = Modifier
                            .size(48.dp)
                            .border(
                                width = 2.dp, color = Color.White, shape = RoundedCornerShape(48.dp)
                            )
                    }
                    IconButton(
                        onClick = {
                            categoryData = index
                        }, modifier = modifier
                    ) {
                        Image(
                            painter = painterResource(id = item),
                            contentDescription = "",
                            modifier = Modifier.fillMaxSize()
                        )
                    }
                    if (index < Constant.imageList.size - 1) {
                        Spacer(modifier = Modifier.width(16.dp))
                    }
                }
            }
            FromLabel("Date")
            TextField(placeholder = {
                Text(
                    "Date", fontSize = 16.sp, modifier = Modifier.alpha(0.7f)
                )
            },
                value = date,
                onValueChange = {
                    date = it
                },
                modifier = Modifier
                    .fillMaxWidth()
                    .height(55.dp),
                shape = RoundedCornerShape(6.dp),
                colors = colors,
                trailingIcon = {
                    Image(
                        painter = painterResource(id = R.mipmap.calendar),
                        contentDescription = "",
                        modifier = Modifier.size(20.dp)
                    )
                })
            FromLabel("Notes")
            TextField(
                placeholder = {
                    Text(
                        "Notes", fontSize = 16.sp, modifier = Modifier.alpha(0.7f)
                    )
                },
                value = state.taskEntity.content,
                maxLines = 5,
                onValueChange = {
                    addViewModel.sendEvent(AddEvent.UpdateContent(it))
                    contentValid = it.isNotEmpty()
                },
                modifier = Modifier
                    .fillMaxWidth()
                    .height(140.dp),
                shape = RoundedCornerShape(6.dp),
                colors = colors,
                isError = !contentValid
            )
            if (!contentValid) {
                Text(text = "Please enter valid text", color = Color.Red)
            }
        }

        Button(
            onClick = {
                if (!titleValid || !contentValid) {
                    ToastUtil.showLongToast("Please enter valid text")
                    return@Button
                }
                DateUtil.getTodayStartLong()
                addViewModel.sendEvent(AddEvent.AddTask(state.taskEntity))
            },
            modifier = Modifier
                .align(Alignment.BottomCenter)
                .padding(start = 16.dp, end = 16.dp, bottom = 24.dp)
                .fillMaxWidth()
                .height(56.dp),
            shape = RoundedCornerShape(50.dp)
        ) {
            Text(text = stringResource(R.string.save))
        }
    }
}