package com.taskttl.ui.viewmodel

import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.compose.LocalLifecycleOwner
import androidx.lifecycle.repeatOnLifecycle
import androidx.lifecycle.viewModelScope
import com.taskttl.data.contract.UiEffect
import com.taskttl.data.contract.UiEvent
import com.taskttl.data.contract.UiState
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.launch

/**
 * 基础视图模型
 * @author devttl
 * @date 2024/08/12
 * @constructor 创建[BaseViewModel]
 */
abstract class BaseViewModel<S : UiState, E : UiEvent, F : UiEffect> : ViewModel() {

    /** 初始状态 */
    private val initialState: S by lazy { initialState() }

    /**
     * 初始状态
     * @return [S]
     */
    protected abstract fun initialState(): S

    /** 用户界面状态 */
    private val _uiState: MutableStateFlow<S> by lazy { MutableStateFlow(initialState) }

    /** 用户界面状态 */
    val uiState: StateFlow<S> by lazy { _uiState }

    /** 用户界面事件 */
    private val _uiEvent: MutableSharedFlow<E> = MutableSharedFlow()

    /** 用户界面效果 */
    private val _uiEffect: MutableSharedFlow<F> = MutableSharedFlow()

    /** 用户界面效果 */
    val uiEffect: Flow<F> = _uiEffect

    /**
     * 处理事件
     * @param [event] 事件
     * @param [state] 状态
     * @return [S?]
     */
    protected abstract suspend fun handleEvent(event: E, state: S): S?

    init {
        subscribeEvents()
    }

    /**
     * 收集事件
     */
    private fun subscribeEvents() {
        viewModelScope.launch {
            _uiEvent.collect {
                reduceEvent(_uiState.value, it)
            }
        }
    }

    /**
     * 发送事件
     * @param [event] 事件
     */
    fun sendEvent(event: E) {
        viewModelScope.launch {
            _uiEvent.emit(event)
        }
    }

    /**
     * 发送状态
     * @param [newState] 新状态
     */
    private fun sendState(newState: S.() -> S) {
        _uiState.value = uiState.value.newState()
    }

    /**
     * 处理事件，更新状态
     * 减少事件
     * @param [state] S
     * @param [event] E
     */
    private fun reduceEvent(state: S, event: E) {
        viewModelScope.launch {
            handleEvent(event, state)?.let { newState -> sendState { newState } }
        }
    }

    /**
     * 发送效果
     * @param [effect] 影响
     */
    protected fun sendEffect(effect: F) {
        viewModelScope.launch { _uiEffect.emit(effect) }
    }
}


/**
 * 收集效果
 * @param [lifecycleState] 生命周期状态
 * @param [sideEffect] 副作用
 */
@Composable
fun <S : UiState, E : UiEvent, F : UiEffect> BaseViewModel<S, E, F>.CollectSideEffect(
    lifecycleState: Lifecycle.State = Lifecycle.State.STARTED,
    sideEffect: (suspend (sideEffect: F) -> Unit),
) {
    val sideEffectFlow = this.uiEffect
    val lifecycleOwner = LocalLifecycleOwner.current

    LaunchedEffect(sideEffectFlow, lifecycleOwner) {
        lifecycleOwner.lifecycle.repeatOnLifecycle(lifecycleState) {
            sideEffectFlow.collect { sideEffect(it) }
        }
    }
}