package com.taskttl.utils

import android.os.Build
import androidx.annotation.RequiresApi
import java.text.SimpleDateFormat
import java.time.Instant
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter
import java.util.Calendar
import java.util.Date
import java.util.Locale

/**
 * 日期工具
 * @author devttl
 * @date 2024/08/14
 */
object DateUtil {
    var YYYY: String = "yyyy"
    var YYYY_MM: String = "yyyy-MM"
    var YYYYMM: String = "yyyyMM"
    var YYYY_MM_DD: String = "yyyy-MM-dd"
    var YYYYMMDD: String = "yyyyMMdd"
    var YYYYMMDDHHMMSS: String = "yyyyMMddHHmmss"
    var HHMMSSSSS: String = "HHmmssSSS"
    var YYYY_MM_DD_HH_MM_SS: String = "yyyy-MM-dd HH:mm:ss"

    /**
     * 获取旧今天开始时间戳 旧接口
     * @return [Long]
     */
    private fun getTodayStartTimestampOld(): Long {
        val calendar = Calendar.getInstance()
        calendar.set(Calendar.HOUR_OF_DAY, 0)
        calendar.set(Calendar.MINUTE, 0)
        calendar.set(Calendar.SECOND, 0)
        calendar.set(Calendar.MILLISECOND, 0)
        return calendar.timeInMillis
    }

    /**
     * 获取今天开始时间戳
     * @return [Long]
     */
    @RequiresApi(Build.VERSION_CODES.O)
    private fun getTodayStartTimestamp(): Long {
        val todayStart = LocalDate.now().atStartOfDay(ZoneId.systemDefault())
        return todayStart.toEpochSecond() * 1000 // 将秒转换为毫秒
    }

    /**
     * 获取今天开始时间戳
     * @return [Long]
     */
    fun getTodayStartLong(): Long {
        val todayStartTimestamp: Long = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            getTodayStartTimestamp()
        } else {
            getTodayStartTimestampOld()
        }
        return todayStartTimestamp
    }

    /**
     * 获取明天开始时间戳 旧接口
     * @return [Long]
     */
    private fun getTomorrowStartTimestampOld(): Long {
        val calendar = Calendar.getInstance()
        calendar.add(Calendar.DAY_OF_YEAR, 1)
        calendar.set(Calendar.HOUR_OF_DAY, 0)
        calendar.set(Calendar.MINUTE, 0)
        calendar.set(Calendar.SECOND, 0)
        calendar.set(Calendar.MILLISECOND, 0)
        return calendar.timeInMillis
    }

    /**
     * 获取明天开始时间戳
     * @return [Long]
     */
    @RequiresApi(Build.VERSION_CODES.O)
    private fun getTomorrowStartTimestamp(): Long {
        val tomorrowStart = LocalDate.now().plusDays(1).atStartOfDay(ZoneId.systemDefault())
        return tomorrowStart.toEpochSecond() * 1000
    }

    /**
     * 获取明天开始时间戳
     * @return [Long]
     */
    fun getTomorrowStartLong(): Long {
        val tomorrowStartTimestamp: Long = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            getTomorrowStartTimestamp()
        } else {
            getTomorrowStartTimestampOld()
        }
        return tomorrowStartTimestamp
    }

    /**
     * 获取时间戳 旧接口
     * @return [Long]
     */
    private fun getTimestampOld(): Long {
        return System.currentTimeMillis()
    }

    /**
     * 获取时间戳
     * @return [Long]
     */
    @RequiresApi(Build.VERSION_CODES.O)
    private fun getTimestamp(): Long {
        return Instant.now().toEpochMilli()
    }

    /**
     * 获取时间戳
     * @return [Long]
     */
    fun getLong(): Long {
        val timestamp: Long = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            getTimestamp()
        } else {
            getTimestampOld()
        }
        return timestamp
    }

    fun today(): String {
        val dateStr: String = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val today = LocalDate.now()
            val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
            today.format(formatter)
        } else {
            val calendar = Calendar.getInstance()
            val sdf = SimpleDateFormat("yyyy-MM-dd", Locale.getDefault())
            sdf.format(calendar)
        }
        return dateStr
    }

    /**
     * 将时间戳转换为日期字符串
     * @param [timestamp] 时间戳
     * @param [format] 格式
     * @return [String]
     */
    @RequiresApi(Build.VERSION_CODES.O)
    private fun convertTimestampToDateString(
        timestamp: Long,
        format: String = YYYY_MM_DD_HH_MM_SS
    ): String {
        val formatter = DateTimeFormatter.ofPattern(format)
        val dateTime =
            LocalDateTime.ofInstant(Instant.ofEpochMilli(timestamp), ZoneId.systemDefault())
        return dateTime.format(formatter)
    }

    /**
     * 将时间戳转换为日期字符串 旧接口
     * @param [timestamp] 时间戳
     * @param [format] 格式
     * @return [String]
     */
    private fun convertTimestampToDateStringOld(
        timestamp: Long,
        format: String = YYYY_MM_DD_HH_MM_SS
    ): String {
        val sdf = SimpleDateFormat(format, Locale.getDefault())
        val date = Date(timestamp)
        return sdf.format(date)
    }

    /**
     * 将时间戳转换为日期 str
     * @param [timestamp] 时间戳
     * @param [format] 格式
     * @return [String]
     */
    fun convertTimestampToDateStr(timestamp: Long, format: String = YYYY_MM_DD_HH_MM_SS): String {
        val dateStr: String = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            convertTimestampToDateString(timestamp, format)
        } else {
            convertTimestampToDateStringOld(timestamp, format)
        }
        return dateStr
    }


}